﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace cAlgo
{
    public class CopierFileService
    {
        private const string TerminalRelativePath = "MetaQuotes\\Terminal";
        private const string TradeCopierFileName = "tradecopier.tc";
        private readonly TradeCopierReceiver _robot;
        private DateTime _fileDoesNotExistSince;
        private bool _fileExists = true;
        private TimeSpan _maxTimeFileMayNotExist;

        public CopierFileService(TradeCopierReceiver robot)
        {
            _robot = robot;
            UpdateTradeCopierFilePath();
            UpdateFileExists();
        }

        public string FilePath { get; private set; }

        public DateTime LastWriteTime { get; private set; }

        public bool CanProcessFile
        {
            get { return _fileExists; }
        }

        private void UpdateTradeCopierFilePath()
        {
            if (CheckExplicitPath())
                return;

            var files = GetCopierFiles().ToArray();
            if (files.Length == 0)
            {
                _robot.Print("Can not file a file. If EA is running, but you still see this message, try to specify Explicit file path");
                _robot.Print("Explicit file path: C:\\Program Files (x86)\\BrokerName MetaTrader 4\\MQL4\\Files\\0_tradecopier.tc");
            }
            else if (files.Length > 1)
            {
                _robot.Print("Found more than one trade copier file with ID: " + _robot.CopierID);
                foreach (var file in files)
                    _robot.Print(file);
            }
            else
            {
                FilePath = files[0];
                _robot.Print("Established connection with trade copier ID: " + _robot.CopierID);
                _robot.Print(files[0]);
            }
        }

        private bool CheckExplicitPath()
        {
            if (string.IsNullOrWhiteSpace(_robot.ExplicitFilePath))
                return false;

            if (File.Exists(_robot.ExplicitFilePath))
            {
                FilePath = _robot.ExplicitFilePath;
                _robot.Print("Established connection with trade copier file: \"" + _robot.ExplicitFilePath + "\"");
                return true;
            }

            _robot.Print("File \"" + _robot.ExplicitFilePath + "\" doesn't exist");
            return true;
        }

        private IEnumerable<string> GetCopierFiles()
        {
            var terminalFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
                                              TerminalRelativePath);

            var expectedTradeCopierFileName = string.Format("{0}_{1}", _robot.CopierID, TradeCopierFileName);

            var files = Directory.GetFiles(terminalFolder, expectedTradeCopierFileName, SearchOption.AllDirectories);
            foreach (var file in files)
                yield return file;

            foreach (var copierFilePath in GetEAPaths(expectedTradeCopierFileName))
                yield return copierFilePath;
        }

        private IEnumerable<string> GetEAPaths(string tradeCopierFileName)
        {
            var specialFolders = new[]
                {Environment.SpecialFolder.ProgramFilesX86, Environment.SpecialFolder.ProgramFiles};

            foreach (var specialFolder in specialFolders)
            {
                var programFilesFolder = Environment.GetFolderPath(specialFolder);
                var metaTraderPaths =
                Directory.GetDirectories(programFilesFolder)
                         .Where(s => Path.GetFileName(s).ToLower().Contains("metatrader"));
                foreach (var metaTraderPath in metaTraderPaths)
                {
                    var eaPath = Path.Combine(metaTraderPath, @"MQL4\Experts",tradeCopierFileName);
                    if (File.Exists(eaPath))
                        yield return eaPath;

                    eaPath = Path.Combine(metaTraderPath, @"MQL4\Files",tradeCopierFileName);
                    if (File.Exists(eaPath))
                        yield return eaPath;
                }
            }
        }

        public void UpdateState()
        {
            UpdateFileExists();

            if (CanProcessFile)
                LastWriteTime = File.GetLastWriteTime(FilePath);
        }

        private void UpdateFileExists()
        {
            var existsNewState = File.Exists(FilePath);
            if (_fileExists != existsNewState)
            {
                if (existsNewState)
                {
                    _robot.Print("Connection established");
                }
                else
                {
                    _fileDoesNotExistSince = _robot.Server.Time;
                }

                _fileExists = existsNewState;
            }
            StopBotIfNeeded();
        }

        private void StopBotIfNeeded()
        {
            if (_fileExists)
                return;

            _maxTimeFileMayNotExist = TimeSpan.FromSeconds(5);
            if (_robot.Server.Time - _fileDoesNotExistSince > _maxTimeFileMayNotExist)
            {
                _robot.Print(string.Format("Couldn't find file for {0}s.", (int) _maxTimeFileMayNotExist.TotalSeconds));
                _robot.Stop();
            }
        }
    }
}